package shcm.shsupercm.fabric.citresewn.cit;

import shcm.shsupercm.fabric.citresewn.CITResewn;
import shcm.shsupercm.fabric.citresewn.api.CITTypeContainer;
import shcm.shsupercm.fabric.citresewn.ex.CITParsingException;
import shcm.shsupercm.fabric.citresewn.pack.format.PropertyGroup;
import shcm.shsupercm.fabric.citresewn.pack.format.PropertyKey;
import shcm.shsupercm.fabric.citresewn.pack.format.PropertyValue;

import java.util.*;
import net.minecraft.class_2960;
import net.minecraft.class_3300;

/**
 * Instanced parent for CIT Types that are applied to items when conditions pass.
 * @see CITTypeContainer
 * @see CIT
 */
public abstract class CITType {
    /**
     * Used to determine which property keys are not conditions.
     * @return a set of property keys used by this type
     */
    public abstract Set<PropertyKey> typeProperties();

    /**
     * Loads the given property group into the type.
     * @param conditions conditions that were parsed out of the property group
     * @param properties group of properties to be read into this type
     * @param resourceManager the CIT's containing resource manager
     * @throws CITParsingException if errored while parsing the type
     */
    public abstract void load(List<CITCondition> conditions, PropertyGroup properties, class_3300 resourceManager) throws CITParsingException;

    protected void warn(String message, PropertyValue value, PropertyGroup properties) {
        CITResewn.logWarnLoading("Warning: " + properties.messageWithDescriptorOf(message, value == null ? -1 : value.position()));
    }

    /**
     * ///// PORTED FROM BETA \\\\\
     * This shit was ported from the
     * beta and will be rewritten at
     * some point!
     * \\\\\                  /////
     *
     * Takes a defined path and resolves it to an identifier pointing to the resourcepack's path of the specified extension(returns null if no path can be resolved).<br>
     * If definedPath is null, will try to resolve a relative file with the same name as the rootIdentifier with the extension, otherwise: <br>
     * definedPath will be formatted to replace "\\" with "/" the extension will be appended if not there already. <br>
     * It will first try using definedPath as an absolute path, if it cant resolve(or definedPath starts with ./), definedPath will be considered relative. <br>
     * Relative paths support going to parent directories using "..".
     */
    public static class_2960 resolveAsset(class_2960 rootIdentifier, String path, String defaultedTypeDirectory, String extension, class_3300 resourceManager) {
        if (path == null) {
            path = rootIdentifier.method_12832().substring(0, rootIdentifier.method_12832().length() - 11);
            if (!path.endsWith(extension))
                path = path + extension;
            class_2960 pathIdentifier = new class_2960(rootIdentifier.method_12836(), path);
            return resourceManager.method_14486(pathIdentifier).isPresent() ? pathIdentifier: null;
        }

        class_2960 pathIdentifier = new class_2960(path);

        path = pathIdentifier.method_12832().replace('\\', '/');
        if (!path.endsWith(extension))
            path = path + extension;

        if (path.startsWith("./"))
            path = path.substring(2);
        else if (!path.contains("..")) {
            pathIdentifier = new class_2960(pathIdentifier.method_12836(), path);
            if (resourceManager.method_14486(pathIdentifier).isPresent())
                return pathIdentifier;
            else if (path.startsWith("assets/")) {
                path = path.substring(7);
                int sep = path.indexOf('/');
                pathIdentifier = new class_2960(path.substring(0, sep), path.substring(sep + 1));
                if (resourceManager.method_14486(pathIdentifier).isPresent())
                    return pathIdentifier;
            }
            pathIdentifier = new class_2960(pathIdentifier.method_12836(), defaultedTypeDirectory + "/" + path);
            if (resourceManager.method_14486(pathIdentifier).isPresent())
                return pathIdentifier;
        }

        LinkedList<String> pathParts = new LinkedList<>(Arrays.asList(rootIdentifier.method_12832().split("/")));
        pathParts.removeLast();

        if (path.contains("/")) {
            for (String part : path.split("/")) {
                if (part.equals("..")) {
                    if (pathParts.size() == 0)
                        return null;
                    pathParts.removeLast();
                } else
                    pathParts.addLast(part);
            }
        } else
            pathParts.addLast(path);
        path = String.join("/", pathParts);

        pathIdentifier = new class_2960(rootIdentifier.method_12836(), path);

        return resourceManager.method_14486(pathIdentifier).isPresent() ? pathIdentifier : null;
    }

    public static class_2960 resolveAsset(class_2960 rootIdentifier, PropertyValue path, String defaultedTypeDirectory, String extension, class_3300 resourceManager) {
        return resolveAsset(rootIdentifier, path == null ? null : path.value(), defaultedTypeDirectory, extension, resourceManager);
    }
}
